@@ -1,4 +1,4 @@
-/* $Id: acconfig.h,v 1.14 2001/02/16 04:48:30 astyanax Exp $ */
+/* $Id: acconfig.h,v 1.15 2001/04/02 05:36:08 astyanax Exp $ */
 
 /* Define this if you have the wresize function in your ncurses-type library */
 #undef HAVE_WRESIZE
@@ -47,3 +47,7 @@
 
 /* Define this to disable the built-in (crappy) file browser */
 #undef DISABLE_BROWSER
+
+/* Define this to disable any and all text wrapping */
+#undef DISABLE_WRAPPING
+
