@@ -1,11 +1,11 @@
-/* $Id: nano.c,v 1.144.2.10 2001/08/17 00:03:40 astyanax Exp $ */
+/* $Id: nano.c,v 1.144.2.18 2001/10/28 14:52:45 astyanax Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
  *   Copyright (C) 1999 Chris Allegretta                                  *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 1, or (at your option)  *
+ *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful,      *
@@ -346,10 +346,12 @@ void align(char **strp)
 }
 
 /* Null a string at a certain index and align it */
-void null_at(char *data, int index)
+void null_at(char **data, int index)
 {
-    data[index] = 0;
-    align(&data);
+
+    /* Ahh!  Damn dereferencing */
+    (*data)[index] = 0;
+    align(data);
 }
 
 void usage(void)
@@ -647,6 +649,7 @@ int do_enter_void(void)
     return do_enter(current);
 }
 
+#ifndef NANO_SMALL
 void do_next_word(void)
 {
     filestruct *fileptr, *old;
@@ -696,8 +699,81 @@ void do_next_word(void)
 
 	update_line(current, current_x);
     }
+}
+/* the same thing for backwards */
+void do_prev_word(void)
+{
+    filestruct *fileptr, *old;
+    int i;
+
+    if (current == NULL)
+	return;
+
+    old = current;
+    i = current_x;
+    for (fileptr = current; fileptr != NULL; fileptr = fileptr->prev) {
+	if (fileptr == current) {
+	    while (isalnum((int) fileptr->data[i])
+		   && i != 0)
+		i--;
+
+	    if (i == 0) {
+		if (fileptr->prev != NULL)
+		    i = strlen(fileptr->prev->data);
+		else if (fileptr == fileage && filebot != NULL) {
+		    current_x = 0;
+		    return;
+		}
+		continue;
+	    }
+	}
+
+	while (!isalnum((int) fileptr->data[i]) && i != 0)
+	    i--;
+
+	if (i > 0) {
+	    i--;
+
+	    while (isalnum((int) fileptr->data[i]) && i != 0)
+		i--;
+
+	    if (!isalnum((int) fileptr->data[i]))
+		i++;
+
+	    if (i != 0 || i != current_x)
+		break;
+
+	}
+	if (fileptr->prev != NULL)
+	    i = strlen(fileptr->prev->data);
+	else if (fileptr == fileage && filebot != NULL) {
+	    current_x = 0;
+	    return;
+	}
+    }
+    if (fileptr == NULL)
+	current = fileage;
+    else
+	current = fileptr;
+
+    current_x = i;
+    placewewant = xplustabs();
+
+    if (current->lineno <= edittop->lineno)
+	edit_update(current, CENTER);
+    else {
+	/* If we've jumped lines, refresh the old line.  We can't just use
+	 * current->prev here, because we may have skipped over some blank
+	 * lines, in which case the previous line is the wrong one.
+	 */
+	if (current != old)
+	    update_line(old, 0);
+
+	update_line(current, current_x);
+    }
 
 }
+#endif /* NANO_SMALL */
 
 #ifndef DISABLE_WRAPPING
 void do_wrap(filestruct * inptr, char input_char)
@@ -828,7 +904,7 @@ void do_wrap(filestruct * inptr, char input_char)
 	    down = 1;
 	}
 
-	null_at(inptr->data, current_x);
+	null_at(&inptr->data, current_x);
 
 	if (ISSET(MARK_ISSET) && (mark_beginbuf == inptr)) {
 	    mark_beginbuf = temp;
@@ -884,7 +960,7 @@ void do_wrap(filestruct * inptr, char input_char)
 		&& (current_x == current_word_start)) {
 		current_x = current_word_start;
 
-		null_at(inptr->data, current_word_start);
+		null_at(&inptr->data, current_word_start);
 	    } else {
 
 		while (isspace((int) inptr->data[i])) {
@@ -2463,6 +2539,13 @@ int main(int argc, char *argv[])
 		modify_control_seq = 1;
 		keyhandled = 1;
 		break;
+#ifndef NANO_SMALL
+	    case ' ':
+		/* If control-space is next word, Alt-space should be previous word */
+		do_prev_word();
+		keyhandled = 1;
+		break;
+#endif
 	    case 91:
 		switch (kbinput = wgetch(edit)) {
 		case '1':	/* Alt-[-1-[0-5,7-9] = F1-F8 in X at least */
@@ -2530,10 +2613,16 @@ int main(int argc, char *argv[])
 		    kbinput = KEY_PPAGE;
 		    wgetch(edit);
 		    break;
+		case 'I':	/* Alt-[-I = Page Up - FreeBSD Console */
+		    kbinput = KEY_PPAGE;
+		    break;
 		case '6':	/* Alt-[-6 = Page Down */
 		    kbinput = KEY_NPAGE;
 		    wgetch(edit);
 		    break;
+		case 'G':	/* Alt-[-G = Page Down - FreeBSD Console */
+		    kbinput = KEY_NPAGE;
+		    break;
 		case '7':
 		    kbinput = KEY_HOME;
 		    wgetch(edit);
@@ -2542,6 +2631,10 @@ int main(int argc, char *argv[])
 		    kbinput = KEY_END;
 		    wgetch(edit);
 		    break;
+		case 'L':		/* Insert Key - FreeBSD Console */
+		    do_insertfile();
+		    keyhandled = 1;
+		    break;
 		case '[':	/* Alt-[-[-[A-E], F1-F5 in linux console */
 		    kbinput = wgetch(edit);
 		    if (kbinput >= 'A' && kbinput <= 'E')
@@ -2644,12 +2737,15 @@ int main(int argc, char *argv[])
 		break;
 #endif
 #endif
+
 	    case 0:		/* Erg */
+#ifndef NANO_SMALL
 		do_next_word();
 		break;
-
+#endif
 	    case 331:		/* Stuff that we don't want to do squat */
 	    case -1:
+	    case 29:		/* Ctrl-] */
 	    case 410:		/* Must ignore this, it gets sent when we resize */
 #ifdef PDCURSES
 	    case 541:		/* ???? */
