@@ -1,11 +1,11 @@
-/* $Id: proto.h,v 1.45.2.1 2001/04/22 07:10:05 astyanax Exp $ */
+/* $Id: proto.h,v 1.45.2.3 2001/10/24 11:42:21 astyanax Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
  *   Copyright (C) 1999 Chris Allegretta                                  *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 1, or (at your option)  *
+ *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful,      *
@@ -131,7 +131,7 @@ void die(char *msg, ...);
 void new_file(void);
 void new_magicline(void);
 void splice_node(filestruct *begin, filestruct *newnode, filestruct *end);
-void null_at(char *data, int index);
+void null_at(char **data, int index);
 void page_up_center(void);
 void blank_edit(void);
 void search_init_globals(void);
